@@ -24,24 +24,29 @@ use crate::{
     BuildContext, Control, HorizontalAlignment, NodeHandleMapping, Orientation, Thickness,
     UserInterface, VerticalAlignment,
 };
+use core::time;
 use std::{
-    cell::RefCell,
+    borrow::BorrowMut,
+    cell,
     ops::{Deref, DerefMut},
     path::{Component, Path, PathBuf, Prefix},
     rc::Rc,
+    sync::{mpsc, Arc, Mutex},
+    thread,
 };
 
 use crate::text_box::TextCommitMode;
+use notify::Watcher;
 use std::fmt::{Debug, Formatter};
 #[cfg(not(target_arch = "wasm32"))]
 use sysinfo::{DiskExt, RefreshKind, SystemExt};
 
 #[derive(Clone)]
-pub struct Filter(pub Rc<RefCell<dyn FnMut(&Path) -> bool>>);
+pub struct Filter(pub Arc<Mutex<dyn FnMut(&Path) -> bool + Send>>);
 
 impl Filter {
-    pub fn new<F: FnMut(&Path) -> bool + 'static>(filter: F) -> Self {
-        Self(Rc::new(RefCell::new(filter)))
+    pub fn new<F: FnMut(&Path) -> bool + 'static + Send>(filter: F) -> Self {
+        Self(Arc::new(Mutex::new(filter)))
     }
 }
 
@@ -75,6 +80,8 @@ pub struct FileBrowser<M: MessageData, C: Control<M, C>> {
     mode: FileBrowserMode,
     file_name: Handle<UINode<M, C>>,
     file_name_value: PathBuf,
+    #[allow(clippy::type_complexity)]
+    watcher: Rc<cell::Cell<Option<(notify::RecommendedWatcher, thread::JoinHandle<()>)>>>,
 }
 
 crate::define_widget_deref!(FileBrowser<M, C>);
@@ -142,15 +149,10 @@ impl<M: MessageData, C: Control<M, C>> Control<M, C> for FileBrowser<M, C> {
                             if message.direction() == MessageDirection::ToWidget
                                 && &self.path != path
                             {
-                                // Skip parts of path until we get existing.
-                                let mut existing_path = path.to_owned();
-                                while !existing_path.exists() {
-                                    if !existing_path.pop() {
-                                        break;
-                                    }
-                                }
+                                let existing_path = ignore_nonexistent_sub_dirs(path);
 
                                 let mut item = find_tree(self.tree_root, &existing_path, ui);
+
                                 if item.is_none() {
                                     // Generate new tree contents.
                                     let result = build_all(
@@ -201,10 +203,27 @@ impl<M: MessageData, C: Control<M, C>> Control<M, C> for FileBrowser<M, C> {
                         }
                         FileBrowserMessage::Root(root) => {
                             if &self.root != root {
+                                let watcher_replacment = match self.watcher.replace(None) {
+                                    Some((mut watcher, converter)) => {
+                                        let current_root = match &self.root {
+                                            Some(path) => path.clone(),
+                                            None => self.path.clone(),
+                                        };
+                                        let _ = watcher.unwatch(current_root);
+                                        let new_root = match &root {
+                                            Some(path) => path.clone(),
+                                            None => self.path.clone(),
+                                        };
+                                        let _ = watcher
+                                            .watch(new_root, notify::RecursiveMode::Recursive);
+                                        Some((watcher, converter))
+                                    }
+                                    None => None,
+                                };
                                 self.root = root.clone();
                                 self.path = root.clone().unwrap_or_default();
-
                                 self.rebuild_from_root(ui);
+                                self.watcher.replace(watcher_replacment);
                             }
                         }
                         FileBrowserMessage::Filter(filter) => {
@@ -212,12 +231,54 @@ impl<M: MessageData, C: Control<M, C>> Control<M, C> for FileBrowser<M, C> {
                                 (Some(current), Some(new)) => std::ptr::eq(&*new, &*current),
                                 _ => false,
                             };
-
                             if !equal {
                                 self.filter = filter.clone();
                                 self.rebuild_from_root(ui);
                             }
                         }
+                        FileBrowserMessage::Add(path) => {
+                            let path =
+                                make_fs_watcher_event_path_relative_to_tree_root(&self.root, path);
+                            if filtered_out(&mut self.filter, &path) {
+                                return;
+                            }
+                            let parent_path = parent_path(&path);
+                            let existing_parent_node = find_tree(self.tree_root, &parent_path, ui);
+                            if existing_parent_node.is_some() {
+                                if let UINode::Tree(tree) = ui.node(existing_parent_node) {
+                                    if tree.expanded() {
+                                        build_tree(
+                                            existing_parent_node,
+                                            existing_parent_node == self.tree_root,
+                                            path,
+                                            parent_path,
+                                            ui,
+                                        );
+                                    } else if !tree.expander_shown() {
+                                        ui.send_message(TreeMessage::set_expander_shown(
+                                            tree.handle(),
+                                            MessageDirection::ToWidget,
+                                            true,
+                                        ))
+                                    }
+                                }
+                            }
+                        }
+                        FileBrowserMessage::Remove(path) => {
+                            let path =
+                                make_fs_watcher_event_path_relative_to_tree_root(&self.root, path);
+                            let node = find_tree(self.tree_root, &path, ui);
+                            if node.is_some() {
+                                let parent_path = parent_path(&path);
+                                let parent_node = find_tree(self.tree_root, &parent_path, ui);
+                                ui.send_message(TreeMessage::remove_item(
+                                    parent_node,
+                                    MessageDirection::ToWidget,
+                                    node,
+                                ))
+                            }
+                        }
+                        FileBrowserMessage::Rescan => (),
                     }
                 }
             }
@@ -250,8 +311,8 @@ impl<M: MessageData, C: Control<M, C>> Control<M, C> for FileBrowser<M, C> {
                     if let Ok(dir_iter) = std::fs::read_dir(&parent_path) {
                         for entry in dir_iter.flatten() {
                             let path = entry.path();
-                            let build = if let Some(filter) = self.filter.as_ref() {
-                                filter.deref().0.borrow_mut().deref_mut()(&path)
+                            let build = if let Some(filter) = self.filter.as_mut() {
+                                filter.0.borrow_mut().deref_mut().lock().unwrap()(&path)
                             } else {
                                 true
                             };
@@ -330,6 +391,46 @@ impl<M: MessageData, C: Control<M, C>> Control<M, C> for FileBrowser<M, C> {
     }
 }
 
+fn parent_path(path: &Path) -> PathBuf {
+    let mut parent_path = path.to_owned();
+    parent_path.pop();
+    parent_path
+}
+
+fn filtered_out(filter: &mut Option<Filter>, path: &Path) -> bool {
+    match filter.as_mut() {
+        Some(filter) => !filter.0.borrow_mut().deref_mut().lock().unwrap()(path),
+        None => false,
+    }
+}
+
+fn ignore_nonexistent_sub_dirs(path: &Path) -> PathBuf {
+    let mut existing_path = path.to_owned();
+    while !existing_path.exists() {
+        if !existing_path.pop() {
+            break;
+        }
+    }
+    existing_path
+}
+
+fn make_fs_watcher_event_path_relative_to_tree_root(
+    root: &Option<PathBuf>,
+    path: &Path,
+) -> PathBuf {
+    match root {
+        Some(ref root) => {
+            let remove_prefix = if *root == PathBuf::from(".") {
+                std::env::current_dir().unwrap()
+            } else {
+                root.clone()
+            };
+            PathBuf::from("./").join(path.strip_prefix(remove_prefix).unwrap_or(path))
+        }
+        None => path.to_owned(),
+    }
+}
+
 fn find_tree<M: MessageData, C: Control<M, C>, P: AsRef<Path>>(
     node: Handle<UINode<M, C>>,
     path: &P,
@@ -341,12 +442,13 @@ fn find_tree<M: MessageData, C: Control<M, C>, P: AsRef<Path>>(
             let tree_path = tree.user_data_ref::<PathBuf>().unwrap();
             if tree_path == path.as_ref() {
                 tree_handle = node;
-            }
-            for &item in tree.items() {
-                let tree = find_tree(item, path, ui);
-                if tree.is_some() {
-                    tree_handle = tree;
-                    break;
+            } else {
+                for &item in tree.items() {
+                    let tree = find_tree(item, path, ui);
+                    if tree.is_some() {
+                        tree_handle = tree;
+                        break;
+                    }
                 }
             }
         }
@@ -397,8 +499,17 @@ fn build_tree<M: MessageData, C: Control<M, C>, P: AsRef<Path>>(
     parent_path: P,
     ui: &mut UserInterface<M, C>,
 ) -> Handle<UINode<M, C>> {
-    let tree = build_tree_item(path, parent_path, &mut ui.build_ctx());
+    let subtree = build_tree_item(path, parent_path, &mut ui.build_ctx());
+    insert_subtree_in_parent(ui, parent, is_parent_root, subtree);
+    subtree
+}
 
+fn insert_subtree_in_parent<M: MessageData, C: Control<M, C>>(
+    ui: &mut UserInterface<M, C>,
+    parent: Handle<UINode<M, C>>,
+    is_parent_root: bool,
+    tree: Handle<UINode<M, C>>,
+) {
     if is_parent_root {
         ui.send_message(TreeRootMessage::add_item(
             parent,
@@ -412,8 +523,6 @@ fn build_tree<M: MessageData, C: Control<M, C>, P: AsRef<Path>>(
             tree,
         ));
     }
-
-    tree
 }
 
 struct BuildResult<M: MessageData, C: Control<M, C>> {
@@ -425,7 +534,7 @@ struct BuildResult<M: MessageData, C: Control<M, C>> {
 fn build_all<M: MessageData, C: Control<M, C>>(
     root: Option<&PathBuf>,
     final_path: &Path,
-    filter: Option<Filter>,
+    mut filter: Option<Filter>,
     ctx: &mut BuildContext<M, C>,
 ) -> BuildResult<M, C> {
     let mut dest_path = PathBuf::new();
@@ -509,10 +618,10 @@ fn build_all<M: MessageData, C: Control<M, C>>(
         if let Ok(dir_iter) = std::fs::read_dir(&full_path) {
             for entry in dir_iter.flatten() {
                 let path = entry.path();
-                if filter
-                    .as_ref()
-                    .map_or(true, |f| f.deref().0.borrow_mut().deref_mut()(&path))
-                {
+                #[allow(clippy::blocks_in_if_conditions)]
+                if filter.as_mut().map_or(true, |f| {
+                    f.0.borrow_mut().deref_mut().lock().unwrap()(&path)
+                }) {
                     let item = build_tree_item(&path, &full_path, ctx);
                     if parent.is_some() {
                         Tree::add_item(parent, item, ctx);
@@ -582,7 +691,7 @@ impl<M: MessageData, C: Control<M, C>> FileBrowserBuilder<M, C> {
     /// during construction stage - there is not enough layout information to do so. You
     /// can send FileBrowserMessage::Path right after creation and it will bring tree item
     /// into view without any problems. It is possible because all widgets were created at
-    /// that moment and layout system can give correct offsets to bring item into view.  
+    /// that moment and layout system can give correct offsets to bring item into view.
     pub fn with_path<P: AsRef<Path>>(mut self, path: P) -> Self {
         self.path = path.as_ref().to_owned();
         self
@@ -721,8 +830,13 @@ impl<M: MessageData, C: Control<M, C>> FileBrowserBuilder<M, C> {
             FileBrowserMode::Open => Default::default(),
         };
 
+        let widget = self.widget_builder.with_child(grid).build();
+        let the_path = match &self.root {
+            Some(path) => path.clone(),
+            _ => self.path.clone(),
+        };
         let browser = FileBrowser {
-            widget: self.widget_builder.with_child(grid).build(),
+            widget,
             tree_root,
             path_text,
             path: match self.mode {
@@ -742,9 +856,67 @@ impl<M: MessageData, C: Control<M, C>> FileBrowserBuilder<M, C> {
             scroll_viewer,
             root: self.root,
             file_name,
+            watcher: Rc::new(cell::Cell::new(None)),
         };
+        let watcher = browser.watcher.clone();
+        let filebrowser_node = UINode::FileBrowser(browser);
+        let node = ctx.add_node(filebrowser_node);
+        watcher.replace(setup_filebrowser_fs_watcher(
+            ctx.ui.sender(),
+            node,
+            the_path,
+        ));
+        node
+    }
+}
 
-        ctx.add_node(UINode::FileBrowser(browser))
+fn setup_filebrowser_fs_watcher<M: MessageData, C: Control<M, C>>(
+    ui_sender: mpsc::Sender<UiMessage<M, C>>,
+    filebrowser_widget_handle: Handle<UINode<M, C>>,
+    the_path: PathBuf,
+) -> Option<(notify::RecommendedWatcher, thread::JoinHandle<()>)> {
+    let (tx, rx) = mpsc::channel();
+    match notify::watcher(tx, time::Duration::from_secs(1)) {
+        Ok(mut watcher) => {
+            #[allow(clippy::while_let_loop)]
+            let watcher_conversion_thread = std::thread::spawn(move || loop {
+                match rx.recv() {
+                    Ok(event) => match event {
+                        notify::DebouncedEvent::Remove(path) => {
+                            let _ = ui_sender.send(FileBrowserMessage::remove(
+                                filebrowser_widget_handle,
+                                MessageDirection::ToWidget,
+                                path,
+                            ));
+                        }
+                        notify::DebouncedEvent::Create(path) => {
+                            let _ = ui_sender.send(FileBrowserMessage::add(
+                                filebrowser_widget_handle,
+                                MessageDirection::ToWidget,
+                                path,
+                            ));
+                        }
+                        notify::DebouncedEvent::Rescan | notify::DebouncedEvent::Error(_, _) => {
+                            let _ = ui_sender.send(FileBrowserMessage::rescan(
+                                filebrowser_widget_handle,
+                                MessageDirection::ToWidget,
+                            ));
+                        }
+                        notify::DebouncedEvent::NoticeRemove(_) => (),
+                        notify::DebouncedEvent::NoticeWrite(_) => (),
+                        notify::DebouncedEvent::Write(_) => (),
+                        notify::DebouncedEvent::Chmod(_) => (),
+                        notify::DebouncedEvent::Rename(_, _) => (),
+                    },
+                    Err(_) => {
+                        break;
+                    }
+                };
+            });
+            let _ = watcher.watch(the_path, notify::RecursiveMode::Recursive);
+            Some((watcher, watcher_conversion_thread))
+        }
+        Err(_) => None,
     }
 }
 
@@ -998,3 +1170,37 @@ impl<M: MessageData, C: Control<M, C>> FileSelectorBuilder<M, C> {
         ctx.add_node(UINode::FileSelector(file_selector))
     }
 }
+
+#[cfg(test)]
+mod test {
+    use crate::{
+        core::{algebra::Vector2, pool::Handle},
+        file_browser::{build_tree, find_tree},
+        node::StubNode,
+        tree::TreeRootBuilder,
+        widget::WidgetBuilder,
+        UserInterface,
+    };
+    use std::{path::PathBuf, rc::Rc};
+
+    #[test]
+    fn test_find_tree() {
+        let mut ui = UserInterface::<(), StubNode>::new(Vector2::new(100.0, 100.0));
+
+        let root = TreeRootBuilder::new(
+            WidgetBuilder::new().with_user_data(Rc::new(PathBuf::from("test"))),
+        )
+        .build(&mut ui.build_ctx());
+
+        let path = build_tree(root, true, "./test/path1", "./test", &mut ui);
+
+        while ui.poll_message().is_some() {}
+
+        // This passes.
+        assert_eq!(find_tree(root, &"./test/path1", &ui), path);
+
+        // This expected to fail
+        // https://github.com/rust-lang/rust/issues/31374
+        assert_eq!(find_tree(root, &"test/path1", &ui), Handle::NONE);
+    }
+}
