@@ -325,6 +325,11 @@ where
     /// Provides a way to respond to OS specific events. Can be useful to detect if a key or mouse
     /// button was pressed. This method significantly differs from `handle_message` because os events
     /// are not dispatched - they'll be passed to this method in any case.
+    ///
+    /// ## Important notes
+    ///
+    /// Due to performance reasons, you **must** set `.with_handle_os_messages(true)` in widget builder to
+    /// force library to call `handle_os_event`!
     fn handle_os_event(
         &mut self,
         _self_handle: Handle<UINode<M, C>>,
@@ -1267,7 +1272,9 @@ impl<M: MessageData, C: Control<M, C>> UserInterface<M, C> {
                 }
 
                 for &handle in self.preview_set.iter() {
-                    self.nodes[handle].preview_message(self, &mut message);
+                    if self.nodes.is_valid_handle(handle) {
+                        self.nodes[handle].preview_message(self, &mut message);
+                    }
                 }
 
                 self.bubble_message(&mut message);
@@ -1357,17 +1364,18 @@ impl<M: MessageData, C: Control<M, C>> UserInterface<M, C> {
                             if *button == MouseButton::Right {
                                 if let Some(picked) = self.nodes.try_borrow(self.picked_node) {
                                     // Get the context menu from the current node or aa parent node
-                                    let context_menu = if picked.context_menu().is_some() {
-                                        picked.context_menu()
+                                    let (context_menu, target) = if picked.context_menu().is_some()
+                                    {
+                                        (picked.context_menu(), self.picked_node)
                                     } else {
                                         let parent_handle = picked.find_by_criteria_up(self, |n| {
                                             n.context_menu().is_some()
                                         });
 
                                         if let Some(parent) = self.nodes.try_borrow(parent_handle) {
-                                            parent.context_menu()
+                                            (parent.context_menu(), parent_handle)
                                         } else {
-                                            Handle::NONE
+                                            (Handle::NONE, Handle::NONE)
                                         }
                                     };
 
@@ -1376,7 +1384,7 @@ impl<M: MessageData, C: Control<M, C>> UserInterface<M, C> {
                                         self.send_message(PopupMessage::placement(
                                             context_menu,
                                             MessageDirection::ToWidget,
-                                            Placement::Cursor,
+                                            Placement::Cursor(target),
                                         ));
                                         self.send_message(PopupMessage::open(
                                             context_menu,
@@ -1705,12 +1713,14 @@ impl<M: MessageData, C: Control<M, C>> UserInterface<M, C> {
         for i in 0..self.nodes.get_capacity() {
             let handle = self.nodes.handle_from_index(i);
 
-            if self.nodes.is_valid_handle(handle) {
-                let (ticket, mut node) = self.nodes.take_reserve(handle);
+            if let Some(node_ref) = self.nodes.try_borrow(handle) {
+                if node_ref.handle_os_events {
+                    let (ticket, mut node) = self.nodes.take_reserve(handle);
 
-                node.handle_os_event(handle, self, event);
+                    node.handle_os_event(handle, self, event);
 
-                self.nodes.put_back(ticket, node);
+                    self.nodes.put_back(ticket, node);
+                }
             }
         }
 
@@ -1850,6 +1860,11 @@ impl<M: MessageData, C: Control<M, C>> UserInterface<M, C> {
         self.nodes.borrow(node_handle)
     }
 
+    #[inline]
+    pub fn try_get_node(&self, node_handle: Handle<UINode<M, C>>) -> Option<&UINode<M, C>> {
+        self.nodes.try_borrow(node_handle)
+    }
+
     pub fn copy_node(&mut self, node: Handle<UINode<M, C>>) -> Handle<UINode<M, C>> {
         let mut map = NodeHandleMapping::default();
 
