@@ -13,6 +13,7 @@
 //! However [WidgetMessage::GotFocus](enum.WidgetMessage.html) has "Direction: From UI" which means that only
 //! internal library code can send such messages without a risk of breaking anything.
 
+use crate::core::algebra::Vector4;
 use crate::{
     brush::Brush,
     core::{
@@ -536,12 +537,16 @@ pub enum DropdownListMessage<M: MessageData, C: Control<M, C>> {
     SelectionChanged(Option<usize>),
     Items(Vec<Handle<UINode<M, C>>>),
     AddItem(Handle<UINode<M, C>>),
+    Open,
+    Close,
 }
 
 impl<M: MessageData, C: Control<M, C>> DropdownListMessage<M, C> {
     define_constructor!(DropdownList(DropdownListMessage:SelectionChanged) => fn selection(Option<usize>), layout: false);
     define_constructor!(DropdownList(DropdownListMessage:Items) => fn items(Vec<Handle<UINode<M, C>>>), layout: false);
     define_constructor!(DropdownList(DropdownListMessage:AddItem) => fn add_item(Handle<UINode<M, C>>), layout: false);
+    define_constructor!(DropdownList(DropdownListMessage:Open) => fn open(), layout: false);
+    define_constructor!(DropdownList(DropdownListMessage:Close) => fn close(), layout: false);
 }
 
 #[derive(Debug, Clone, PartialEq)]
@@ -549,14 +554,14 @@ pub enum PopupMessage<M: MessageData, C: Control<M, C>> {
     Open,
     Close,
     Content(Handle<UINode<M, C>>),
-    Placement(Placement),
+    Placement(Placement<M, C>),
 }
 
 impl<M: MessageData, C: Control<M, C>> PopupMessage<M, C> {
     define_constructor!(Popup(PopupMessage:Open) => fn open(), layout: false);
     define_constructor!(Popup(PopupMessage:Close) => fn close(), layout: false);
     define_constructor!(Popup(PopupMessage:Content) => fn content(Handle<UINode<M, C>>), layout: false);
-    define_constructor!(Popup(PopupMessage:Placement) => fn placement(Placement), layout: false);
+    define_constructor!(Popup(PopupMessage:Placement) => fn placement(Placement<M, C>), layout: false);
 }
 
 #[derive(Debug, Clone, PartialEq)]
@@ -597,16 +602,18 @@ pub enum TreeMessage<M: MessageData, C: Control<M, C>> {
     },
     AddItem(Handle<UINode<M, C>>),
     RemoveItem(Handle<UINode<M, C>>),
+    SetExpanderShown(bool),
     SetItems(Vec<Handle<UINode<M, C>>>),
     // Private, do not use. For internal needs only. Use TreeRootMessage::Selected.
     Select(SelectionState),
 }
 
 impl<M: MessageData, C: Control<M, C>> TreeMessage<M, C> {
+    define_constructor!(Tree(TreeMessage:Expand) => fn expand(expand: bool, expansion_strategy: TreeExpansionStrategy), layout: false);
     define_constructor!(Tree(TreeMessage:AddItem) => fn add_item(Handle<UINode<M, C>>), layout: false);
     define_constructor!(Tree(TreeMessage:RemoveItem) => fn remove_item(Handle<UINode<M, C>>), layout: false);
+    define_constructor!(Tree(TreeMessage:SetExpanderShown) => fn set_expander_shown(bool), layout: false);
     define_constructor!(Tree(TreeMessage:SetItems) => fn set_items(Vec<Handle<UINode<M, C>>>), layout: false);
-    define_constructor!(Tree(TreeMessage:Expand) => fn expand(expand: bool, expansion_strategy: TreeExpansionStrategy), layout: false);
 
     pub(in crate) fn select(
         destination: Handle<UINode<M, C>>,
@@ -648,12 +655,18 @@ pub enum FileBrowserMessage {
     Root(Option<PathBuf>),
     Path(PathBuf),
     Filter(Option<Filter>),
+    Add(PathBuf),
+    Remove(PathBuf),
+    Rescan,
 }
 
 impl FileBrowserMessage {
     define_constructor_unbound!(FileBrowser(FileBrowserMessage:Root) => fn root(Option<PathBuf>), layout: false);
     define_constructor_unbound!(FileBrowser(FileBrowserMessage:Path) => fn path(PathBuf), layout: false);
     define_constructor_unbound!(FileBrowser(FileBrowserMessage:Filter) => fn filter(Option<Filter>), layout: false);
+    define_constructor_unbound!(FileBrowser(FileBrowserMessage:Add) => fn add(PathBuf), layout: false);
+    define_constructor_unbound!(FileBrowser(FileBrowserMessage:Remove) => fn remove(PathBuf), layout: false);
+    define_constructor_unbound!(FileBrowser(FileBrowserMessage:Rescan) => fn rescan(), layout: false);
 }
 
 #[derive(Debug, Clone, PartialEq)]
@@ -738,6 +751,15 @@ impl NumericUpDownMessage {
     define_constructor_unbound!(NumericUpDown(NumericUpDownMessage:Value) => fn value(f32), layout: false);
 }
 
+#[derive(Debug, Clone, PartialEq)]
+pub enum Vec2EditorMessage {
+    Value(Vector2<f32>),
+}
+
+impl Vec2EditorMessage {
+    define_constructor_unbound!(Vec2Editor(Vec2EditorMessage:Value) => fn value(Vector2<f32>), layout: false);
+}
+
 #[derive(Debug, Clone, PartialEq)]
 pub enum Vec3EditorMessage {
     Value(Vector3<f32>),
@@ -747,6 +769,15 @@ impl Vec3EditorMessage {
     define_constructor_unbound!(Vec3Editor(Vec3EditorMessage:Value) => fn value(Vector3<f32>), layout: false);
 }
 
+#[derive(Debug, Clone, PartialEq)]
+pub enum Vec4EditorMessage {
+    Value(Vector4<f32>),
+}
+
+impl Vec4EditorMessage {
+    define_constructor_unbound!(Vec4Editor(Vec4EditorMessage:Value) => fn value(Vector4<f32>), layout: false);
+}
+
 #[derive(Debug, Clone, PartialEq)]
 pub enum ScrollPanelMessage<M: MessageData, C: Control<M, C>> {
     VerticalScroll(f32),
@@ -943,7 +974,9 @@ pub enum UiMessageData<M: MessageData, C: Control<M, C>> {
     FileSelector(FileSelectorMessage),
     TextBox(TextBoxMessage),
     NumericUpDown(NumericUpDownMessage),
+    Vec2Editor(Vec2EditorMessage),
     Vec3Editor(Vec3EditorMessage),
+    Vec4Editor(Vec4EditorMessage),
     Menu(MenuMessage),
     MenuItem(MenuItemMessage),
     MessageBox(MessageBoxMessage),
@@ -997,7 +1030,7 @@ impl MessageDirection {
     }
 }
 
-pub trait MessageData: 'static + Debug + Clone + PartialEq {}
+pub trait MessageData: 'static + Debug + Clone + PartialEq + Send {}
 
 /// Message is basic communication element that is used to deliver information to UI nodes
 /// or to user code.
